from flask import request, session, Blueprint, Response, redirect
from htmlpy import *
from config import CONFIG
from os.path import exists
import os
import datetime
import gnupg

main_pages = Blueprint("main", __name__)

BOOTSTRAP = [
    Reference(
        "https://cdn.jsdelivr.net/npm/bootstrap@5.2.0-beta1/dist/css/bootstrap.min.css",
        "stylesheet",
    ),
    Reference(
        "https://cdn.jsdelivr.net/npm/bootstrap-icons@1.9.1/font/bootstrap-icons.css",
        "stylesheet",
    ),
    Script(
        src="https://cdn.jsdelivr.net/npm/bootstrap@5.2.0-beta1/dist/js/bootstrap.bundle.min.js"
    ),
]


def encrypt(msg):
    pgp = gnupg.GPG()
    pgp.import_keys(open("/config/pub.key").read())
    return str(pgp.encrypt(msg, pgp.list_keys()[0]["fingerprint"]))


def save_message(msg, name=""):
    os.makedirs("/data/messages", exist_ok=True)
    dt = datetime.datetime.now().strftime("%Y-%m-%d.%H-%M")
    f = open(f"/data/messages/{name}-{dt}.asc", "w")
    f.write(msg)


@main_pages.route("/message", methods=["GET", "POST"])
def send_message():
    if request.method == "POST":
        msg = request.form["message"]
        name = request.form["msg_name"]
        if msg is not None:
            cipher = encrypt(msg)
            save_message(cipher, name)
            return redirect(request.url_root)
    else:
        return buildSite(
            [
                Div(
                    [
                        Heading(1, "Message"),
                        LineBreak(),
                        Form(
                            destination=request.url,
                            inner=[
                                Input(
                                    "",
                                    placeholder="Name",
                                    name="msg_name",
                                    global_attr=GlobalAttributes(
                                        css_class="form-control bg-dark text-white",
                                        style="margin-bottom: 15px",
                                    ),
                                ),
                                TextArea(
                                    inner="",
                                    placeholder="Message",
                                    name="message",
                                    global_attr=GlobalAttributes(
                                        css_class="form-control bg-dark text-white",
                                        style="margin-bottom: 15px;",
                                    ),
                                ),
                                Input(
                                    type=InputType.Submit,
                                    value="Send Message",
                                    name="submit",
                                    global_attr=GlobalAttributes(
                                        css_class="btn btn-danger text-white text-decoration-none"
                                    ),
                                ),
                            ],
                        ),
                    ],
                    global_attr=GlobalAttributes(
                        css_class="container", style="margin-top: 25px"
                    ),
                )
            ]
        ).to_code()


def buildSite(content):
    return Document(
        head=Head(
            [
                Meta(
                    name="viewport",
                    content="user-scalable=no, width=device-width, initial-scale=1.0",
                ),
                BOOTSTRAP,
            ]
        ),
        body=Body(
            content,
            global_attr=GlobalAttributes(
                css_class="bg-dark text-white justify-content-center text-center"
            ),
        ),
    )


@main_pages.route("/public_key", methods=["GET"])
def public_key():
    try:
        ret = open("/config/pub.key").read()

        ua = request.user_agent.string.lower()
        if "chrome" in ua or "safari" in ua or "firefox" in ua:
            return buildSite(
                [
                    Div(
                        Div(
                            [
                                Bold("To Import: "),
                                Span(
                                    f'curl -sL "{request.base_url}"|gpg --import',
                                    global_attr=GlobalAttributes(
                                        style="display: block;font-family: monospace,monospace;margin-top: 10px; font-size: 20px;overflow-wrap: break-word;"
                                    ),
                                ),
                            ],
                            global_attr=GlobalAttributes(css_class="alert alert-info"),
                        ),
                        global_attr=GlobalAttributes(
                            css_class="container", style="margin-top: 25px"
                        ),
                    ),
                    Div(
                        Paragraph(ret.replace("\n", "<br>")),
                        global_attr=GlobalAttributes(
                            css_class="container card bg-primary"
                        ),
                    ),
                ]
            ).to_code()

        resp = Response(response=ret, status=200, mimetype="application/pgp-keys")
        return resp
    except:
        return Response(response="", status=502)


def build_contact_block():
    return Div(
        [
            Heading(1, "Contact"),
            ThematicBreak(),
            [
                Link("/public_key", "My PGP Key"),
                LineBreak(),
                Link("/message", "Write a message"),
                LineBreak(),
                LineBreak(),
            ]
            if exists("/config/pub.key")
            else None,
            Link(f"mailto:{CONFIG['email']}", CONFIG["email"]),
            LineBreak(),
        ],
        global_attr=GlobalAttributes(css_class="container border-dark"),
    )


def build_donation_block():
    return Div(
        [
            Heading(1, "Donation"),
            ThematicBreak(),
            Paragraph(
                [
                    Bold("Monero: "),
                    Span(
                        CONFIG["xmr_address"],
                        global_attr=GlobalAttributes(
                            style="color: orange;overflow-wrap: break-word;"
                        ),
                    ),
                ]
            )
            if "xmr_address" in CONFIG
            else None,
        ],
        global_attr=GlobalAttributes(css_class="container", style="margin-top: 20px"),
    )


def build_information_block():
    return Div(
        [
            Image(
                "/assets/me",
                200,
                200,
                "Me",
                global_attr=GlobalAttributes(css_class="rounded"),
            ),
            LineBreak(),
            LineBreak(),
            Heading(1, CONFIG["name"]),
            ThematicBreak(),
        ],
        global_attr=GlobalAttributes(
            css_class="container border-dark", style="margin-top: 20px"
        ),
    )


# Main
@main_pages.route("/", methods=["GET"])
def index():
    return buildSite(
        [build_information_block(), build_contact_block(), build_donation_block()]
    ).to_code()
